@@ -20,7 +20,8 @@
 #
 
 # Std Lib
-import sys, struct
+import sys
+import struct
 
 # Local
 from g import *
@@ -359,6 +360,7 @@ PRINT_INTERNAL_PAGE_COLOR_CAL_VERIFICATION = 1104
 PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE = 1100 # LBOW/OJ Pro L7xxx
 PRINT_INTERNAL_PAGE_ALIGNMENT_PAGE_VERIFICATION = 1150 # LBOW
 PRINT_INTERNAL_PAGE_LINEFEED_CALIBRATION = 1407 # OJ Pro
+PRINT_INTERNAL_PAGE_AUTOMATIC_COLOR_CALIBRATION = 1408 # PS Pro B8800
 PRINT_INTERNAL_PAGE_PRINT_QUALITY_DIAGNOSTIC = 1409 # OJ Pro
 
 # From xojpanel
@@ -510,6 +512,8 @@ POWER_MODE_CHARGING = 0x04
 POWER_MODE_DISCHARGING = 0x08
 POWER_MODE_BATTERY_LEVEL_KNOWN = 0x10
 
+OID_BATTERY_LEVEL_2 = ('1.1.2.61', TYPE_SIGNED_INTEGER)
+
 OID_POWER_SETTINGS = ('1.1.2.118', TYPE_ENUMERATION)
 OID_POWER_SETTINGS_15MIN = 1
 OID_POWER_SETTINGS_30MIN = 2
